﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2011/ 1/20 初版
//
// FVC05 Continue Grab Sample Program
// -----------------------------------------------------------------
namespace Form_FVC05_CONTGRABSAMPLE
{
    public partial class Form_FVC05_CONTGRABSAMPLE : Form
    {
        //CH0用画像メモリ格納領域
        private List<FVIL.Data.CFviImage> m_image_CH0 = new List<FVIL.Data.CFviImage>();
        //CH1用画像メモリ格納領域
        private List<FVIL.Data.CFviImage> m_image_CH1 = new List<FVIL.Data.CFviImage>();
        //2値化CH0用画像メモリ格納領域
        private List<FVIL.Data.CFviImage> m_image_CH0BIN = new List<FVIL.Data.CFviImage>();
        //2値化CH1用画像メモリ格納領域
        private List<FVIL.Data.CFviImage> m_image_CH1BIN = new List<FVIL.Data.CFviImage>();
        //FVC05クラス
        private FVIL.Video.CFviVideoFVC05 m_video = new FVIL.Video.CFviVideoFVC05();
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        //画像オブジェクト番号
        public int m_intBufImgIndex;
        //iniファイル名（フルパス）
        public string m_strIniFilePath;
        //ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;
        //フリーランの状態
        private bool m_flgFreerun;

        //FVC05ローカルバッファ使用時の定数
        const uint FVC05_LOCALMEM_SIZE = 0x01000000;     //16MB
        const uint LOCAL_BUFFER_BOUNDARY = 0x40000;      //256KB
        const uint LOCAL_BUFFER_MASK = 0x3FFFF;          //(LOCAL_BUFFER_BOUNDARY - 1)

        public Form_FVC05_CONTGRABSAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            //使用許諾の確認
            dialogLicence licence_dialog = new dialogLicence();
            if (licence_dialog.ShowDialog(this) != DialogResult.OK)
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

             //iniファイル選択
            fnSetIniFile();

        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。

            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideoOpened = false;
            m_flgFreerun = false;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0.000";
            //フレーム数テキストボックス初期化
            TextBoxFrame.Text = "0";
            //キャプチャー数テキストボックス初期化
            TextBoxCaptureNumber.Text = "10";
            //表示画像メモリ番号初期化
            TextBoxBufImgIndex.Text = "0";
            //ローカルバッファ確保最大枚数初期化
            TextBoxMaxImgNum.Text = "0";
            //ローカルバッファ確保枚数初期化
            NumericUpDownBufConfigNum.Value = 0;

        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FVC051CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                //ロックされたメモリを解除します。
                if (m_video.ContinuousGrab.IsMemoryLocked())
                {
                    //取り込みの停止
                    m_video.ContinuousGrab.StopCapture();
                    //画像メモリロックの解除
                    m_video.ContinuousGrab.MemoryUnlock();
                }
                // ビデオデバイスがオープンされていた場合はクローズする
                if (m_flgVideoOpened)
                {
                    m_video.Close();
                    m_flgVideoOpened = false;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            Form_BROWSEIN formbroseini = new Form_BROWSEIN();
            //iniファイル設定画面

            //iniファイルの選択
            m_strIniFilePath = "";
            if (formbroseini.ShowDialog(this) == DialogResult.OK)
            {
                m_strIniFilePath = formbroseini.strIniFilePath;
            }

            if (m_strIniFilePath.Length > 0)
            {
                try
                {
                    //ビデオデバイスがオープンされていた場合はクローズする
                    if (m_video.IsOpen())
                    {
                        m_video.Close();
                        m_flgVideoOpened = false;
                    }

                    //デバイスオープン
                    m_video.Open(-1, 0);
                    m_flgVideoOpened = true;

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);

                    //２カメラ同時取り込みに設定
                    m_video.SimGrab = 2;

                    //iniファイルをコンポーネントに設定
                    if ((m_strIniFilePath.Length > 0))
                    {
                        m_video.LoadIniFile(m_strIniFilePath);
                    }

                    //各チャネルのステータスを取得
                    //ビデオ入力オフセット(CH0)の取得
                    NumericUpDownOffsetCH0.Value = m_video.VideoOffsetCH0;
                    //２値化閾値(CH0)の取得
                    NumericUpDownThresholdCH0.Value = m_video.BinarizeCH0.Threshold;
                    //ビデオ入力オフセット(CH1)の取得
                    NumericUpDownOffsetCH1.Value = m_video.VideoOffsetCH1;
                    //２値化閾値(CH1)の取得
                    NumericUpDownThresholdCH1.Value = m_video.BinarizeCH1.Threshold;
                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            double msec = 0;
            bool status;

            try
            {
                if (m_video.ContinuousGrab.IsMemoryLocked())
                {
                    //画像メモリがロックされていたら開放する
                    m_video.ContinuousGrab.MemoryUnlock();
                }

                //ローカルバッファ割り当て設定
                fnSetBufConfig();

                //画像メモリロック
                status = fnMemLock(1);
                if (status == false)
                {
                    //フリーラン停止処理
                    fnStopFreerun();
                    return;
                }

                //時間測定開始
                m_timer.Start();
                //カメラ取り込み
                m_video.ContinuousGrab.GrabSync(1);
                //時間測定終了
                msec = m_timer.Stop();
                //測定時間表示
                TextBoxTime.Text = msec.ToString("0.000");
                //再描画
                CFviImageViewCH0.Refresh();
                CFviImageViewCH1.Refresh();
                CFviImageViewCH0BIN.Refresh();
                CFviImageViewCH1BIN.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Grab Continueボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// フリーラン開始・停止。
        /// </remarks>
        private void ButtonGrabContinue_Click(object sender, EventArgs e)
        {
            bool status;

            //ボタン表示変更
            if (m_flgFreerun == true)
            {
                //フリーラン停止処理
                fnStopFreerun();
            }
            else
            {
                try
                {
                    //計測時間テキストボックス初期化
                    TextBoxTime.Text = "0.000";
                    //フレーム数テキストボックス初期化
                    TextBoxFrame.Text = "0";
                    ButtonGrabContinue.Text = "Stop";
                    ButtonGrabOne.Enabled = false;
                    ButtonLeft.Enabled = false;
                    ButtonRight.Enabled = false;
                    NumericUpDownLockImgNum.Enabled = false;

                    if (m_video.ContinuousGrab.IsMemoryLocked())
                    {
                        //画像メモリがロックされていたら開放する
                        m_video.ContinuousGrab.MemoryUnlock();
                    }

                    //ローカルバッファ割り当て設定
                    fnSetBufConfig();

                    //画像メモリロック
                    status = fnMemLock((uint)NumericUpDownLockImgNum.Value);
                    if (status == false)
                    {
                        //フリーラン停止処理
                        fnStopFreerun();
                        return;
                    }

                    //フリーラン開始処理
                    m_flgFreerun = true;
                    timer1.Interval = 50;
                    timer1.Start();
                    //連続カメラ取り込み
                    m_video.ContinuousGrab.GrabASync(uint.Parse(TextBoxCaptureNumber.Text));
                }
                catch (FVIL.CFviException ex)
                {
                    //フリーラン停止処理
                    fnStopFreerun();
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //フリーラン停止処理
                    fnStopFreerun();
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            Application.DoEvents();

        }

        /// <summary>
        /// フリーラン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 連続カメラ取り込み。
        /// </remarks>
        private void timer1_Tick(object sender, EventArgs e)
        {
            timer1.Stop();

            try
            {
                if (m_flgFreerun == true)
                {
                    //フレーム数表示更新
                    TextBoxFrame.Text = m_video.ContinuousGrab.QueryCaptureNo().TotalFrameCount.ToString();

                    //最新フレームをセット
                    if (CheckBoxCH0Gray.Checked == true)
                    {
                        CFviImageViewCH0.Image = m_image_CH0[(int)m_video.ContinuousGrab.QueryCaptureNo().LastFrameNo];
                    } else {
                        CFviImageViewCH0.Image = null;
                    }
                    if (CheckBoxCH1Gray.Checked == true)
                    {
                        CFviImageViewCH1.Image = m_image_CH1[(int)m_video.ContinuousGrab.QueryCaptureNo().LastFrameNo];
                    } else {
                        CFviImageViewCH1.Image = null;
                    }
                    if (CheckBoxCH0Bin.Checked == true)
                    {
                        CFviImageViewCH0BIN.Image = m_image_CH0BIN[(int)m_video.ContinuousGrab.QueryCaptureNo().LastFrameNo];
                    } else {
                        CFviImageViewCH0BIN.Image = null;
                    }
                    if (CheckBoxCH1Bin.Checked == true)
                    {
                        CFviImageViewCH1BIN.Image = m_image_CH1BIN[(int)m_video.ContinuousGrab.QueryCaptureNo().LastFrameNo];
                    } else {
                        CFviImageViewCH1BIN.Image = null;
                    }
                    
                    //再描画
                    CFviImageViewCH0.Refresh();
                    CFviImageViewCH1.Refresh();
                    CFviImageViewCH0BIN.Refresh();
                    CFviImageViewCH1BIN.Refresh();
                    //フリーラン継続・停止判定
                    if ((uint.Parse(TextBoxCaptureNumber.Text) > m_video.ContinuousGrab.QueryCaptureNo().TotalFrameCount) || (uint.Parse(TextBoxCaptureNumber.Text) == 0))
                    {
                        timer1.Start();
                    } else {
                        //フリーラン停止処理
                        fnStopFreerun();
                    }
                }
                timer1.Enabled = true;
            }
            catch (FVIL.CFviException ex)
            {
                //フリーラン停止処理
                fnStopFreerun();
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //フリーラン停止処理
                fnStopFreerun();
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// ローカルバッファ割り当て設定関数
        /// </summary>
        /// <remarks>
        /// ローカルバッファ割り当て設定を変更する。
        /// </remarks>
        private void fnSetBufConfig()
        {
            FVIL.Video.CFviFBufConfigNum Channel = new FVIL.Video.CFviFBufConfigNum();
            Channel.Channel = 0;
            Channel.Num = (uint)NumericUpDownBufConfigNum.Value;

            if (CheckBoxCH0Gray.Checked == true)
            {
                Channel.ChannelGray0 = true;
            }
            if (CheckBoxCH1Gray.Checked == true)
            {
                Channel.ChannelGray1 = true;
            }
            if (CheckBoxCH0Bin.Checked == true)
            {
                Channel.ChannelBin0 = true;
            }
            if (CheckBoxCH1Bin.Checked == true)
            {
                Channel.ChannelBin1 = true;
            }
            m_video.FBufConfigNum = Channel;
        }

        /// <summary>
        /// 画像メモリロック関数
        /// </summary>
        /// <remarks>
        /// 画像メモリをロックする。
        /// </remarks>
        private bool fnMemLock(uint LockImgNum)
        {
            try
            {
                int LockNum;
                //メモリ格納領域クリア
                m_image_CH0.Clear();
                m_image_CH1.Clear();
                m_image_CH0BIN.Clear();
                m_image_CH1BIN.Clear();
                if (CheckBoxCH0Gray.Checked == true)
                {
                    for(LockNum = 0; LockNum < LockImgNum; LockNum++)
                    {
                        //画像メモリ確保数分メモリ格納領域追加
                        m_image_CH0.Add(new FVIL.Data.CFviImage((int)m_video.ImageHorz, (int)m_video.ImageVert, FVIL.ImageType.UC8, 1));
                    }
                    //連続取り込み先の画像管理クラスのコレクション設定
                    m_video.ContinuousGrab.Gray0Images = m_image_CH0;
                    //描画imageのセット
                    CFviImageViewCH0.Image = m_image_CH0[0];
                } else {
                    //連続取り込み先の画像管理クラスのコレクション設定
                    m_video.ContinuousGrab.Gray0Images = null;
                    //描画imageのセット
                    CFviImageViewCH0.Image = null;
                }
                if (CheckBoxCH1Gray.Checked == true)
                {
                    for(LockNum = 0; LockNum < LockImgNum; LockNum++ )
                    {
                        //画像メモリ確保数分メモリ格納領域追加
                        m_image_CH1.Add(new FVIL.Data.CFviImage((int)m_video.ImageHorz, (int)m_video.ImageVert, FVIL.ImageType.UC8, 1));
                    }
                    //連続取り込み先の画像管理クラスのコレクション設定
                    m_video.ContinuousGrab.Gray1Images = m_image_CH1;
                    //描画imageのセット
                    CFviImageViewCH1.Image = m_image_CH1[0];
                } else {
                    //連続取り込み先の画像管理クラスのコレクション設定
                    m_video.ContinuousGrab.Gray1Images = null;
                    //描画imageのセット
                    CFviImageViewCH1.Image = null;
                }
                if (CheckBoxCH0Bin.Checked == true)
                {
                    for (LockNum = 0; LockNum < LockImgNum; LockNum++ )
                    {
                        //画像メモリ確保数分メモリ格納領域追加
                        m_image_CH0BIN.Add(new FVIL.Data.CFviImage((int)m_video.ImageHorz, (int)m_video.ImageVert, FVIL.ImageType.BIN, 1));
                    }
                    //連続取り込み先の画像管理クラスのコレクション設定
                    m_video.ContinuousGrab.Bin0Images = m_image_CH0BIN;
                    //描画imageのセット
                    CFviImageViewCH0BIN.Image = m_image_CH0BIN[0];
                } else {
                    //連続取り込み先の画像管理クラスのコレクション設定
                    m_video.ContinuousGrab.Bin0Images = null;
                    //描画imageのセット
                    CFviImageViewCH0BIN.Image = null;
                }
                if (CheckBoxCH1Bin.Checked == true)
                {
                    for (LockNum = 0; LockNum < LockImgNum; LockNum++)
                    {
                        //画像メモリ確保数分メモリ格納領域追加
                        m_image_CH1BIN.Add(new FVIL.Data.CFviImage((int)m_video.ImageHorz, (int)m_video.ImageVert, FVIL.ImageType.BIN, 1));
                    }
                    //連続取り込み先の画像管理クラスのコレクション設定
                    m_video.ContinuousGrab.Bin1Images = m_image_CH1BIN;
                    //描画imageのセット
                    CFviImageViewCH1BIN.Image = m_image_CH1BIN[0];
                } else {
                    //連続取り込み先の画像管理クラスのコレクション設定
                    m_video.ContinuousGrab.Bin1Images = null;
                    //描画imageのセット
                    CFviImageViewCH1BIN.Image = null;
                }
                //画像メモリロック
                m_video.ContinuousGrab.MemoryLock(LockImgNum);
                return (true);
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }
        }

        /// <summary>
        /// フリーラン停止関数
        /// </summary>
        /// <remarks>
        /// フリーラン停止処理と各種ステータスを変更する。
        /// </remarks>
        private void fnStopFreerun()
        {
            try
            {
                timer1.Stop();
                timer1.Enabled = false;
                //画像取り込みの停止()
                m_video.ContinuousGrab.StopCapture();
                m_flgFreerun = false;
                ButtonGrabContinue.Text = "Grab Continue";
                ButtonGrabOne.Enabled = true;
                ButtonLeft.Enabled = true;
                ButtonRight.Enabled = true;
                NumericUpDownLockImgNum.Enabled = true;
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// ←ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像メモリの番号を戻す。
        /// </remarks>
        private void ButtonLeft_Click(object sender, EventArgs e)
        {
            int size;

            m_intBufImgIndex = int.Parse(TextBoxBufImgIndex.Text) - 1;
            size = (int)NumericUpDownLockImgNum.Value;
            if (m_intBufImgIndex < 0)
            {
                m_intBufImgIndex = size - 1;
            }
            if (m_intBufImgIndex < 0)
            {
                m_intBufImgIndex = 0;
            }

            TextBoxBufImgIndex.Text = m_intBufImgIndex.ToString();
            if (m_image_CH0.Count > 0)
            {
                //表示画像をセット
                CFviImageViewCH0.Image = m_image_CH0[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                CFviImageViewCH0.Refresh();
            }
            if (m_image_CH1.Count > 0)
            {
                //表示画像をセット
                CFviImageViewCH1.Image = m_image_CH1[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                CFviImageViewCH1.Refresh();
            }
            if (m_image_CH0BIN.Count > 0)
            {
                //表示画像をセット
                CFviImageViewCH0BIN.Image = m_image_CH0BIN[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                CFviImageViewCH0BIN.Refresh();
            }
            if (m_image_CH1BIN.Count > 0)
            {
                //表示画像をセット
                CFviImageViewCH1BIN.Image = m_image_CH1BIN[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                CFviImageViewCH1BIN.Refresh();
            }

        }

        /// <summary>
        /// →ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像メモリの番号を進める。
        /// </remarks>
        private void ButtonRight_Click(object sender, EventArgs e)
        {
            int size;

            m_intBufImgIndex = int.Parse(TextBoxBufImgIndex.Text) + 1;
            size = (int)NumericUpDownLockImgNum.Value;
            if (m_intBufImgIndex >= size)
            {
                m_intBufImgIndex = 0;
            }

            TextBoxBufImgIndex.Text = m_intBufImgIndex.ToString();
            if (m_image_CH0.Count > 0)
            {
                //表示画像をセット
                CFviImageViewCH0.Image = m_image_CH0[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                CFviImageViewCH0.Refresh();
            }
            if (m_image_CH1.Count > 0)
            {
                //表示画像をセット
                CFviImageViewCH1.Image = m_image_CH1[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                CFviImageViewCH1.Refresh();
            }
            if (m_image_CH0BIN.Count > 0)
            {
                //表示画像をセット
                CFviImageViewCH0BIN.Image = m_image_CH0BIN[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                CFviImageViewCH0BIN.Refresh();
            }
            if (m_image_CH1BIN.Count > 0)
            {
                //表示画像をセット
                CFviImageViewCH1BIN.Image = m_image_CH1BIN[int.Parse(TextBoxBufImgIndex.Text)];
                //再描画
                CFviImageViewCH1BIN.Refresh();
            }

        }

        /// <summary>
        /// Video Offset CH0値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// オフセット(CH0)の設定。
        /// </remarks>
        private void NumericUpDownOffsetCH0_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    //チャネル０に設定
                    m_video.VideoOffsetCH0 = (int)NumericUpDownOffsetCH0.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Video Offset CH1値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// オフセット(CH1)の設定。
        /// </remarks>
        private void NumericUpDownOffsetCH1_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    //チャネル１に設定
                    m_video.VideoOffsetCH1 = (int)NumericUpDownOffsetCH1.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Bin Threshold CH0値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 2値化しきい値(CH0)の設定。
        /// </remarks>
        private void NumericUpDownThresholdCH0_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviVideoBinarize bin = m_video.Binarize;
                    bin.Threshold = (uint)NumericUpDownThresholdCH0.Value;
                    //チャネル０に設定
                    m_video.BinarizeCH0 = bin;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Bin Threshold CH1値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ２値化しきい値(CH1)の設定。
        /// </remarks>
        private void NumericUpDownThresholdCH1_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    FVIL.Video.CFviVideoBinarize bin = m_video.Binarize;
                    bin.Threshold = (uint)NumericUpDownThresholdCH1.Value;
                    //チャネル１に設定
                    m_video.BinarizeCH1 = bin;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// x1ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります

            //表示倍率の設定。
            CFviImageViewCH0.Display.Magnification = 1.0;
            CFviImageViewCH1.Display.Magnification = 1.0;
            CFviImageViewCH0BIN.Display.Magnification = 1.0;
            CFviImageViewCH1BIN.Display.Magnification = 1.0;
            //再描画
            CFviImageViewCH0.Refresh();
            CFviImageViewCH1.Refresh();
            CFviImageViewCH0BIN.Refresh();
            CFviImageViewCH1BIN.Refresh();
        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            CFviImageViewCH0.FitImageSize();
            CFviImageViewCH1.FitImageSize();
            CFviImageViewCH0BIN.FitImageSize();
            CFviImageViewCH1BIN.FitImageSize();
            //再描画
            CFviImageViewCH0.Refresh();
            CFviImageViewCH1.Refresh();
            CFviImageViewCH0BIN.Refresh();
            CFviImageViewCH1BIN.Refresh();
        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = CFviImageViewCH0.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            CFviImageViewCH0.Display.Magnification = Value;
            CFviImageViewCH1.Display.Magnification = Value;
            CFviImageViewCH0BIN.Display.Magnification = Value;
            CFviImageViewCH1BIN.Display.Magnification = Value;
            //再描画
            CFviImageViewCH0.Refresh();
            CFviImageViewCH1.Refresh();
            CFviImageViewCH0BIN.Refresh();
            CFviImageViewCH1BIN.Refresh();
        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = CFviImageViewCH0.Display.Magnification;
            Value = Value - 0.5;
            if ((Value <= 0))
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            CFviImageViewCH0.Display.Magnification = Value;
            CFviImageViewCH1.Display.Magnification = Value;
            CFviImageViewCH0BIN.Display.Magnification = Value;
            CFviImageViewCH1BIN.Display.Magnification = Value;
            //再描画
            CFviImageViewCH0.Refresh();
            CFviImageViewCH1.Refresh();
            CFviImageViewCH0BIN.Refresh();
            CFviImageViewCH1BIN.Refresh();
        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                CFviImageViewCH0.Display.OverlayAscope.Enable = true;
                CFviImageViewCH0.Display.OverlayAscope.Active = true;
                CFviImageViewCH1.Display.OverlayAscope.Enable = true;
                CFviImageViewCH1.Display.OverlayAscope.Active = true;
                CFviImageViewCH0BIN.Display.OverlayAscope.Enable = true;
                CFviImageViewCH0BIN.Display.OverlayAscope.Active = true;
                CFviImageViewCH1BIN.Display.OverlayAscope.Enable = true;
                CFviImageViewCH1BIN.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                CFviImageViewCH0.Display.OverlayAscope.Enable = false;
                CFviImageViewCH0.Display.OverlayAscope.Active = false;
                CFviImageViewCH1.Display.OverlayAscope.Enable = false;
                CFviImageViewCH1.Display.OverlayAscope.Active = false;
                CFviImageViewCH0BIN.Display.OverlayAscope.Enable = false;
                CFviImageViewCH0BIN.Display.OverlayAscope.Active = false;
                CFviImageViewCH1BIN.Display.OverlayAscope.Enable = false;
                CFviImageViewCH1BIN.Display.OverlayAscope.Active = false;
            }
            //再描画
            CFviImageViewCH0.Refresh();
            CFviImageViewCH1.Refresh();
            CFviImageViewCH0BIN.Refresh();
            CFviImageViewCH1BIN.Refresh();
        }

        /// <summary>
        /// ReLoad Ini Fileボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ビデオ設定用iniファイルの再設定。
        /// </remarks>
        private void ButtonReLoadIni_Click(object sender, EventArgs e)
        {
            //フリーラン中ならば、停止する。
	        if (m_flgFreerun == true) {
                //フリーラン停止処理
                fnStopFreerun();
		        Application.DoEvents();
	        }

        	//iniファイル選択
	        fnSetIniFile();
        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    String name;
                    char[] ex;
                    int temp;
                    name = ".";
                    ex = name.ToCharArray();
                    //拡張子までの文字数を算出
                    temp = ofdlg.FileName.Length - ofdlg.FileName.LastIndexOfAny(ex);
                    temp = ofdlg.FileName.Length - temp;

                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    if (CheckBoxCH0Gray.Checked == true)
                    {
                        //チャネル０のグレイ画像保存
                        name = ofdlg.FileName.Insert(temp, "_0Ch");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewCH1.Image);
                    }
                    if (CheckBoxCH0Gray.Checked == true)
                    {
                        //チャネル１のグレイ画像保存
                        name = ofdlg.FileName.Insert(temp, "_1Ch");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewCH1.Image);
                    }
                    if (CheckBoxCH0Bin.Checked == true)
                    {
                        //チャネル０の二値画像保存
                        name = ofdlg.FileName.Insert(temp, "_0ChBin");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewCH0BIN.Image);
                    }
                    if (CheckBoxCH1Bin.Checked == true)
                    {
                        //チャネル１の二値画像保存
                        name = ofdlg.FileName.Insert(temp, "_1ChBin");
                        FVIL.File.Function.SaveImageFile(name, CFviImageViewCH1BIN.Image);
                    }

                    //保存するファイル形式が固定の場合は、以下のように記述します。
                    //例1) bmpファイルの場合
                    //Dim file As FVIL.File.CFviImageFileBmp = New FVIL.File.CFviImageFileBmp()
                    //file.Save(ofdlg.FileName, m_image)

                    //例2) jpgファイルの場合
                    //Dim file As FVIL.File.CFviImageFileJpg = New FVIL.File.CFviImageFileJpg()
                    //file.Save(ofdlg.FileName, m_image)
                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch (ex.ErrorCode)
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// キャプチャー数テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// キャプチャー数のクランプ処理。
        /// </remarks>
        private void TextBoxCaptureNumber_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (int.Parse(TextBoxCaptureNumber.Text) < 0)
                {
                    TextBoxCaptureNumber.Text = "0";
                }
            }
            catch (Exception)
            {
                //エラー処理
                //数字以外の入力は10に初期化する。
                TextBoxCaptureNumber.Text = "10";
            }
        }

        /// <summary>
        /// 取込み画像選択チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// チャネル０のグレイ画像選択
        /// </remarks>
        private void CheckBoxCH0Gray_CheckedChanged(object sender, EventArgs e)
        {
            //最大確保枚数の再計算
            fnCalcMaxImgNum();
        }

        /// <summary>
        /// 取込み画像選択チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// チャネル１のグレイ画像選択
        /// </remarks>
        private void CheckBoxCH1Gray_CheckedChanged(object sender, EventArgs e)
        {
            //最大確保枚数の再計算
            fnCalcMaxImgNum();
        }

        /// <summary>
        /// 取込み画像選択チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// チャネル０の２値画像選択
        /// </remarks>
        private void CheckBoxCH0Bin_CheckedChanged(object sender, EventArgs e)
        {
            //最大確保枚数の再計算
            fnCalcMaxImgNum();
        }

        /// <summary>
        /// 取込み画像選択チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// チャネル１の２値画像選択
        /// </remarks>
        private void CheckBoxCH1Bin_CheckedChanged(object sender, EventArgs e)
        {
            //最大確保枚数の再計算
            fnCalcMaxImgNum();
        }

        /// <summary>
        /// ローカルバッファ確保最大枚数計算関数
        /// </summary>
        /// <remarks>
        /// 確保可能最大枚数を算出する
        /// </remarks>
        private void fnCalcMaxImgNum()
        {
            uint GrayImgSize;
            uint BinImgSize;

            GrayImgSize = m_video.ImageHorz * m_video.ImageVert;
            BinImgSize = GrayImgSize / 8;

            if ((CheckBoxCH0Gray.Checked == false) && (CheckBoxCH1Gray.Checked == false) && (CheckBoxCH0Bin.Checked == false) && (CheckBoxCH1Bin.Checked == false))
            {
                TextBoxMaxImgNum.Text = "0";
                NumericUpDownBufConfigNum.Value = 0;
            } else {
                uint ImgSize = 0;
                uint TotalSize = 0;
                uint Count = 0;
                uint frame_size;

                if (CheckBoxCH0Gray.Checked == true)
                {
                    ImgSize = ImgSize + GrayImgSize;
                }
                if (CheckBoxCH1Gray.Checked == true)
                {
                    ImgSize = ImgSize + GrayImgSize;
                }
                if (CheckBoxCH0Bin.Checked == true)
                {
                    ImgSize = ImgSize + BinImgSize;
                }
                if (CheckBoxCH1Bin.Checked == true)
                {
                    ImgSize = ImgSize + BinImgSize;
                }
                Count = FVC05_LOCALMEM_SIZE / ImgSize;
                for (uint i = Count; i > 0; i--)
                {
                    if (((i * GrayImgSize) & LOCAL_BUFFER_MASK) > 0)
                    {
                        frame_size = ((i * GrayImgSize) & (~LOCAL_BUFFER_MASK)) + LOCAL_BUFFER_BOUNDARY;
                    } else {
                        frame_size = (i * GrayImgSize);
                    }
                    if (CheckBoxCH0Gray.Checked == true)
                    {
                        TotalSize = TotalSize + frame_size;
                    }
                    if (CheckBoxCH1Gray.Checked == true)
                    {
                        TotalSize = TotalSize + frame_size;
                    }
                    if (((i * BinImgSize) & LOCAL_BUFFER_MASK) > 0)
                    {
                        frame_size = ((i * BinImgSize) & (~LOCAL_BUFFER_MASK)) + LOCAL_BUFFER_BOUNDARY;
                    } else {
                        frame_size = (i * BinImgSize);
                    }
                    if (CheckBoxCH0Bin.Checked == true)
                    {
                        TotalSize = TotalSize + frame_size;
                    }
                    if (CheckBoxCH1Bin.Checked == true)
                    {
                        TotalSize = TotalSize + frame_size;
                    }
                    if (TotalSize <= FVC05_LOCALMEM_SIZE)
                    {
                        Count = i;
                        break;
                    } else {
                        TotalSize = 0;
                    }
                }
                TextBoxMaxImgNum.Text = Count.ToString();
                NumericUpDownBufConfigNum.Value = Count;
            }
        }

        /// <summary>
        /// ローカルバッファ確保枚数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ローカルバッファ確保枚数のクランプ処理
        /// </remarks>
        private void NumericUpDownBufConfigNum_ValueChanged(object sender, EventArgs e)
        {
            if (NumericUpDownBufConfigNum.Value > int.Parse(TextBoxMaxImgNum.Text))
            {
                NumericUpDownBufConfigNum.Value = int.Parse(TextBoxMaxImgNum.Text);
            }

        }

        /// <summary>
        /// ローカルバッファ確保枚数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ローカルバッファ確保枚数のクランプ処理
        /// </remarks>
        private void NumericUpDownBufConfigNum_KeyUp(object sender, KeyEventArgs e)
        {
            if (NumericUpDownBufConfigNum.Value > int.Parse(TextBoxMaxImgNum.Text))
            {
                NumericUpDownBufConfigNum.Value = int.Parse(TextBoxMaxImgNum.Text);
            }

        }

    }
}